local QTC = LibStub("LibQTipClick-1.1")
local LDB = LibStub("LibDataBroker-1.1")
BES = LibStub("AceAddon-3.0"):NewAddon("Broker_BlizzardEquipmentSet", "AceConsole-3.0", "AceTimer-3.0", "AceEvent-3.0")

--------------------------------------------------------------------------------
-- Tooltip functions
local function Tooltip_ClickCell(event, cell, arg, button)
	if arg ~= "§§close" then
		UseEquipmentSet(arg)
	end
	if QTC:IsAcquired("Broker_BlizzEquipmentSet") then QTC:Release(BES.tooltip) end
end

local function Tooltip_Show(frame)
	if QTC:IsAcquired("Broker_BlizzEquipmentSet") then QTC:Release(BES.tooltip) end
	BES.tooltip = QTC:Acquire("Broker_BlizzEquipmentSet", 2, "LEFT", "LEFT")
	-- Assign the callback to the tooltip
	BES.tooltip:SetCallback("OnMouseDown", Tooltip_ClickCell)
	
	for i = 1, GetNumEquipmentSets() do
		name, icon = GetEquipmentSetInfo(i)
		local y, x = BES.tooltip:AddLine()
		y, x = BES.tooltip:SetCell(y, 1, "|T" .. icon ..":16|t", name)
		y, x = BES.tooltip:SetCell(y, 2, name, name)
	end
	
	local y, x = BES.tooltip:AddLine()
	y, x = BES.tooltip:SetCell(y, 1, "", "§§close")
	y, x = BES.tooltip:SetCell(y, 2, "Close", "§§close")
	
	BES.tooltip:SmartAnchorTo(frame)
	BES.tooltip:Show()
end

local function Tooltip_Hide()
	-- Release the tooltip
	BES.tooltip:Hide()
	QTC:Release(BES.tooltip)
end

--------------------------------------------------------------------------------
-- Ace addon events
function BES:OnInitialize()
    -- Called when the addon is loaded
    self.lDBObject = {
		type = "data source",
		label = "BlizzEquipmentSet",
		icon = "Interface\\Icons\\INV_Misc_QuestionMark",
		OnEnter = Tooltip_Show,
		OnClick = Tooltip_Hide,
	}
	
	LDB:NewDataObject("BlizzEquipmentSet", self.lDBObject)
end

function BES:OnEnable()
    -- Called when the addon is enabled
    self:RegisterEvent("PLAYER_ENTERING_WORLD", "UpdateLDBObject")
    self:RegisterEvent("UNIT_INVENTORY_CHANGED", "OnInventoryChanged")
end

function BES:OnDisable()
    -- Called when the addon is disabled
end

function BES:OnInventoryChanged(event, unit)
	if unit=="player" then
		self:UpdateLDBObject()
	end
end

local function matchesToEquipped(setName)
	local equipmentSetItemIds = GetEquipmentSetItemIDs(setName)
	local matches = true
	for slotId, itemId in pairs(equipmentSetItemIds) do
		local itemLink = GetInventoryItemLink("player", slotId)
		if itemLink == nil then
			if itemId ~= 0 then matches = false end
		else
			local found, _, itemLinkString = string.find(itemLink, "^|c%x+|H(.+)|h%[.*%]")
			if not found then
				matches = false
			else
				local _, itemLinkId = strsplit(":", itemLinkString)
				if 0+itemLinkId ~= itemId then
					matches = false
				end
			end
		end
	end
	return matches
end

function BES:UpdateLDBObject()
	for setIndex = 1, GetNumEquipmentSets() do
		local setName, setIcon = GetEquipmentSetInfo(setIndex)
		if matchesToEquipped(setName) then
			self.lDBObject.text = setName
			self.lDBObject.icon = setIcon
			break
		else
			self.lDBObject.text = "Custom set"
			self.lDBObject.icon = "Interface\\Icons\\INV_Misc_QuestionMark"
		end
	end
	
	-- Update LDB object with icon
	BES.equipmentSets = equipmentSets
end
